#!/bin/sh
# --------------------------------------------------------------
# learnspam.sh v1.6
# Train SpamAssassin with its sa-learn tool & optionally report
# spam to Vipul's Razor.
#
# 03/2005  - Original script released
# ...time passes many undocumented changes...
# 05/05/05 - v1.2 Modified sed command from 's/ \+/ /g' to
#            's/ \{1,\}/ g' because the former does not work
#            on FreeBSD. Thanks to w0ls0n in #qmr for noticing
#            this and to my_haz in #sed for helping me to debug
#            it.
# 05/06/05 - v1.3 Added SA_LEARN and RIPMIME variables instead
#            of calling the binaries without paths. This SHOULD
#            been done from the start, was probably overlooked
#            in haste.
# 05/09/05 - v1.4 Added MKTEMP variable for mktemp binary just
#            in case.
# 05/09/05 - v1.5 Added RIPMIME_RECURSE variable
#
# 05/06/05 - v1.6 Added some more logging. Modified SA
#            logging tocapture stderr & stdout
#
# Requires:
#          o RipMIME from http://www.pldaniels.com/ripmime
#          o SpamAssassin
#          o A thisisspam@yourdomain.tld /Maildir/ style email
#            account on your server
#          o Users must forward AS ATTACHMENTS spam that
#            SpamAssassin mis-identified as HAM
#            TO: thisisspam@yourdomain.tld
#          o *Optional* An account for Vipul's Razor
#
# May be run as a cron job or manually
#
# Sample crontab entry to run once per hour, on the hour:
#
# 0 * * * * vpopmail /path/to/script/learnspam.sh > /dev/null 2>&1
# Originally created by UniCav on freenode.net's #qmr channel.
# Cavin Greer - <CavinGreerATnatchezofficed0tcom>
#
# Modified by me, mtnbkr on #qmr
# William Arlofski - <mtnbkr-qmrATrevpold0tcom>
#
# Thank you UniCav for the original concept and a starting
# point, and thank you #qmr members for the additional
# motivation and testing!
#
# ** Use at your own peril **
#
# -------------------------------------------------------------
 
# Set some semi-secure permissions
# --------------------------------
umask 007
 
# Set some variables
# ------------------
DOMAIN="domain.tld"
SPAMUSER="thisisspam"
MAILDIR="/home/vpopmail/domains/$DOMAIN/$SPAMUSER/Maildir/new"
SA_LEARN_LOG="/var/log/salearn.log"
DATESTR="+%Y-%m-%d %H:%M:%S"
 
# Specifiy system spcific binaries
# --------------------------------
SA="/usr/bin/spamassassin"
SA_LEARN="/usr/bin/sa-learn"
RIPMIME="/usr/local/bin/ripmime"
MKTEMP="/bin/mktemp"
 
# -------------------------------------------------------------
# Comment this out once "sa-learn --dump magic" shows more than
# the recommended minimum of 1000 known spams. Until then, this
# script will run "sa-learn --dump magic" and parse for this
# value each time it is run and --forget the emails before
# attempting to re-learn them as --spam
#
NEWINSTALL="1000"
#
#
# Change this to YES to include reporting to Vipul's Razor. You
# will need to create an account with Vipul's Razor and test
# manually before enabling this option. See the online
# documentation at http://razor.sourceforge.net/docs/
#
REPORT_TO_RAZOR="YES"
#
#
# Set this to the number of levels deep you want ripmime to
# recurse when unpacking emails. man ripmime for more info
#
RIPMIME_RECURSE="2"
# -------------------------------------------------------------
# THERE SHOULD BE NO REASON TO CHANGE ANYTHIG BELOW HERE
# IF THERE IS, PLEASE LET ME KNOW SO IT MAY BE ADDED TO THE
# MAIN DISTRIBUTION OF THIS SCRIPT - mtnbkr
# -------------------------------------------------------------
 
 
    # Write a startup time stamp to the log
    # -------------------------------------
    echo `date "$DATESTR"`" - Starting script" >> "$SA_LEARN_LOG"
 
    # Check to see if there are any emails in
    # the INBOX.
    # ---------------------------------------
    if [ `ls "$MAILDIR" | wc -l` -ge 1 ]; then
 
    # Let's get silly with logging
    # ----------------------------
    echo `date "$DATESTR"`" - Found" `ls "$MAILDIR" | wc -l` "message(s) to process" >> "$SA_LEARN_LOG"
 
     # Create temp directory
     # ---------------------
     echo  -n `date "$DATESTR"`" - Creating temp directory: " >> "$SA_LEARN_LOG"
     TMPDIR=`"$MKTEMP" -d /tmp/learnspam-XXXXXX 2>/dev/null`
     echo "$TMPDIR" >> "$SA_LEARN_LOG"
 
     # Break apart forwarded spam emails with ripmime
     # and remove the email if ripmime exited cleanly
     # ----------------------------------------------
     for email in `ls "$MAILDIR"`; do
      echo `date "$DATESTR"`" - ripmiming email: $email" >> "$SA_LEARN_LOG"
 
      # If ripmime exits with an error, let's leave
      # that email alone for future inspection
      # Currently ripmime seems to be ALWYS exiting
      # with a zero exit status so this does not
      # work. The if loop may be completly removed
      # if you want to blindly delete email that
      # ripmime had problems with.
      # -------------------------------------------
      if "$RIPMIME" --paranoid --recursion-max "$RIPMIME_RECURSE" \
         -i "$MAILDIR/$email" -d "$TMPDIR"; then
       rm -f "$MAILDIR/$email"
      fi
     done
 
     # Delete the textfile[01369]* in the temp
     # directory, leaving only the 'full message'
     # files, and textfile2* files if microsoft
     # outlook was involved
     # ------------------------------------------
     echo `date "$DATESTR"`" - Removing textfile[01369] from $TMPDIR" >> "$SA_LEARN_LOG"
     rm -f "$TMPDIR"/textfile[01369]*
 
     # If this is a new install, and nspam < $NEWINSTALL, first
     # forget the email, then we re-learn it in the next block
     # --------------------------------------------------------
     if [ x"$NEWINSTALL" != "x" ]; then
      if [ `"$SA_LEARN" --dump magic | grep nspam \
         | sed 's/ \{1,\}/ /g' | cut -d' ' -f3` -le "$NEWINSTALL" ]; then
       msg="Known spams is still < $NEWINSTALL. Forgetting misreported hams."
       echo `date "$DATESTR"`" - $msg" >> "$SA_LEARN_LOG"
       echo -n `date "$DATESTR"`" - sa-learn --forget: " >> "$SA_LEARN_LOG"
       "$SA_LEARN" --forget "$TMPDIR"/* >> "$SA_LEARN_LOG" 2>&1
      fi
     fi
 
     # Run sa-learn on spam attachments
     # and also report them to Vipul's Razor
     # -------------------------------------
     echo -n `date "$DATESTR"`" - sa-learn --spam: " >> "$SA_LEARN_LOG"
     "$SA_LEARN" --spam "$TMPDIR"/* >> "$SA_LEARN_LOG" 2>&1
     if [ "$REPORT_TO_RAZOR" = "YES" ]; then
      echo -n `date "$DATESTR"`" - spamassassin -x -r : " >> "$SA_LEARN_LOG"
      cat "$TMPDIR"/* | "$SA" -x -r >> "$SA_LEARN_LOG" 2>&1
     fi
 
     # Clean/remove temp directory
     # ---------------------------
     echo `date "$DATESTR"`" - Removing temp dir: $TMPDIR" >> "$SA_LEARN_LOG"
     rm -rf "$TMPDIR"
 
     # Log a finshed line
     # ------------------
     echo `date "$DATESTR"`" - Finished." >> "$SA_LEARN_LOG"
 
    else
     echo `date "$DATESTR"`" - No spams to learn. Exiting." >> "$SA_LEARN_LOG"
    fi
